/* c-iter.c    v0.1  (C) 1999 adolfo@di-mare.com */

/* compile and link together:
    c-iter+list       ==> main && container
    forwl+backl+order ==> iterators          */

#include "list.h"
#include "iterator.h"
#include "forwl.h"
#include "backl.h"
#include "order.h"

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>

void traverse (list *L, iterator(list)* I);
void print_list(list* L);

void traverse (list *L, iterator(list)* I) {
    for (I->bind(I, L); !I->finished(I); I->next(I)) {
        long *v = (long*) list_retrieve(L, I->here(I));
        printf(" %ld", *v);
    }
} /* traverse */

void main () {
    unsigned long memInit = coreleft();
    list          L;

    list_init(L, long);

    {   int i;
        srand(0);
        printf("\n\n ==== loading ======\n\n");
        for (i=1; i<=23; i++) {
            long val;
            val = i;
            val = rand();
            list_append(&L, &val);
            printf(" %ld", val);
        }
    }

    {   lpos p;
        printf("\n\n ==== nextnext======\n\n");
        for (p = list_first(&L); NULL != p; p = list_next(&L,p)) {
            long *v = (long*) list_retrieve(L, p);
            printf(" %ld", *v);
        }
    }

    {   list_forward  Iforw;
        init_forward(&Iforw);
            printf("\n\n ==== forward ======\n");
            traverse(&L, &Iforw.vmt);
        done_forward( &Iforw.vmt );
    }

    print_list(&L);

    {   int  longcmp(const void *a, const void *b);

        list_order  Iorder;
        init_order(&Iorder, longcmp);
            printf("\n\n ==== order ======\n");
            traverse(&L, &Iorder.vmt);
        done_order   (&Iorder.vmt );
    }

    {
    #ifdef BACKL_H
        list_backward  Iback;
        init_backward(&Iback);
            printf("\n\n ==== backward ======\n");
            traverse(&L, &Iback.vmt);
        done_backward(&Iback.vmt );
    #endif
    }

    list_done(&L);

    if (memInit != coreleft()) {
        printf("\n ==== memory allocation error ====\n");
    };
}

void print_list(list* L) {
    list_forward  Iforw;
    init_forward(&Iforw);

    printf("\n\n ==== forward ======\n");

    bind_forward(&Iforw.vmt, L);
    (Iforw.vmt).bind(&(Iforw.vmt), L);

    for (
       itr_bind(Iforw, *L);    /* i = 0 */
       !itr(Iforw,finished);   /* i < n */
       itr(Iforw,next)         /*  i++  */
    ) {
        long *v = (long*) list_retrieve(L, itr(Iforw, here));
        printf(" %ld",  *v);
    }
    done_forward( &Iforw.vmt );
}

int longcmp(const void *a, const void *b) {
    if ( *((long*)a) < *((long*)b) ) {
        return -1;
    } else if (*((long*)a) > *((long*)b)) {
        return 1;
    } else {
        return 1;
    }
}

/* EOF: c-iter.c */
