/* list.c      v0.1  (C) 1999 adolfo@di-mare.com */

#include "list.h"

#include <mem.h>
#include <stdlib.h>

typedef list_node node;

#define lpos_node(p) ((node*)p)

void list_init_(list *L, size_t sz) {
    L->_node_size = sz + offsetof(list_node, val);
    L->_first = NULL;
}

void list_done(list *L) { /* destructor */
    while (NULL != lpos_node(L->_first)) {
        node *next = lpos_node(L->_first)->next;
        /* a better implementation would invoke
           the destructor node->~val() */
        free(L->_first);
        lpos_node(L->_first) = next;
    }
}

size_t list_count(list *L) {
    list_node *p = (list_node*) (L->_first);
    size_t count = 0;
    while (NULL != p) {
        p = p->next;
        count++;
    }

    return count;
}

void list_insert(list *L, lpos p, void *val) {
/* Puts "val" after position "p" in list "L"
   - Not trully generic: copies "*val" with
     memcpy() instead of val.Operator=()    */

    /* allocate just enough storage for "*val" */
    node  * new_node;
    new_node = (node *) malloc(L->_node_size);

    /* should use val.Operator=() */
    memcpy(
        &new_node->val,
        val,
        L->_node_size-offsetof(list_node, val));

    if (NULL == p) {
        new_node->next = (node*)(L->_first);
        L->_first      = (lpos) new_node;
    } else {
        new_node->next   = ((node*)p) -> next;
        ((node*)p)->next = new_node;
    }

}

lpos list_last(list *L) {
    node *last = (node*) (L->_first);
    if (NULL != last) {
        while (NULL != last->next) {
            last = last->next;
        }
    }

    return (lpos) last;
}

#ifdef COMPILE_NON_INLINE_METHODS
/* Non-macro versions of inlined code */

void list_append_(list *L, void *val) {
/* inserts at the end of the list */
    list_insert(L, list_last(L), val);
}

lpos list_first_(list *L) {
    return L->_first;
}

void* list_retrieve_(lpos p) {
    #pragma argsused
    return list_retrieve(L,p);
}

lpos list_next_(list *L, lpos p) {
    #pragma argsused
    return ((lpos) ( ((list_node*)p)->next));
}

#endif

/* EOF:   list.c */
