/* order.c     v0.1  (C) 1999 adolfo@di-mare.com */

#include "order.h"
#include <mem.h>
#include <alloc.h>

void init_order(
   list_order *I,
   int (*fcmp)(const void *, const void *)
) {
    memcpy(I->mark, "Init", 12);
    I->vmt.bind     = bind_order;
    I->vmt.finished = finished_order;
    I->vmt.here     = here_order;
    I->vmt.next     = next_order;
    I->vmt.done     = done_order;

    I->L = NULL;
    I->v = NULL;
    I->fcmp = fcmp;
}

void bind_order( iterator(list) *I_vmt , list *L) {
    list_order *I = vmt_self(list_order, I_vmt);
    memcpy(I->mark, "bind", 12);

    if (NULL != I->v) {
       free(I->v);
    }

    I->index = 0;
    I->count = list_count(L);
    if (0 == I->count) {
        I->v = NULL;
        return;
    }

    I->L = L;
    I->v = (lpos *) malloc(I->count * sizeof(lpos));
    {   int  i = I->count;
        lpos p = list_first(L);
        while (i>0) {
            i--;
            I->v[i] = p;
            p = list_next(L, p);
        }
    }

    {   int    changed;
        size_t j, k, N = I->count;
        k = 0;
        do {             /* yep: bubble sort! */
            changed = 0;
            for (j = 0; (j < N-k); j++) {
                void *vj     = list_retrieve(L, I->v[j]);
                void *vjplus = list_retrieve(L, I->v[j+1]);

                if (I->fcmp(vj, vjplus) > 0) {
                      lpos tmp = I->v[j];
                      I->v[j] =  I->v[j+1];
                      I->v[j+1] = tmp;
                      changed = 1;
                }
            }
            k++;
        } while (0 != changed);
    }
}

int finished_order( iterator(list) *I_vmt ) {
    list_order *I = vmt_self(list_order, I_vmt);
    memcpy(I->mark, "finished", 12);

    return (NULL == I->v);
}

lpos here_order( iterator(list) *I_vmt ) {
    list_order *I = vmt_self(list_order, I_vmt);
    memcpy(I->mark, "here", 12);

    return I->v[I->index];
}

void next_order( iterator(list) *I_vmt ) {
    list_order *I = vmt_self(list_order, I_vmt);
    memcpy(I->mark,  "next", 12);

    I->index++;
    if (I->index >= I->count) {
       free(I->v);
       I->v = NULL;
    }
}

void done_order( iterator(list) *I_vmt ) {
    list_order *I = vmt_self(list_order, I_vmt);
    memcpy(I->mark,  "done", 12);

    if (NULL != I->v) {
       free(I->v);
    }
}

/* EOF: order.c */
